"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const communication_1 = __importDefault(require("../communication/communication"));
const python_helper_1 = require("./python-helper");
function getId() {
    return 'id_' + Date.now() + parseInt(String(Math.random() * 100000));
}
class Python {
    constructor() {
        this.statements = [];
        this.nA = (arg) => {
            return Object.assign(Object.assign({}, arg), { __pipcook__args: true });
        };
        this.import = (packageName) => {
            const identifier = getId();
            this.statements.push(`import ${packageName} as ${identifier}`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        this.install = (packageName, version) => {
            this.statements.push(`!pip install ${packageName}${version ? '==' + version : ''}`);
        };
        this.buildin = (functionName, object) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${functionName}(${object.__pipcook__identifier})`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        this.createNumber = (number) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${number}`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        this.createString = (string, isRaw = false) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${isRaw ? 'r' : ''}'${string}'`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        this.createBoolean = (value) => {
            const identifier = getId();
            if (value) {
                this.statements.push(`${identifier} = True`);
            }
            else {
                this.statements.push(`${identifier} = False`);
            }
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        this.createNone = () => {
            const identifier = getId();
            this.statements.push(`${identifier} = None`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        this.createTuple = (value) => {
            const identifier = getId();
            this.statements.push(`${identifier} = (${value})`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        this.createList = (value) => {
            const identifier = getId();
            this.statements.push(`${identifier} = [${value}]`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        this.createSet = (value) => {
            const identifier = getId();
            this.statements.push(`${identifier} = set((${value}))`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        this.createDictionary = (value) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${JSON.stringify(value)}`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            return pythonObject;
        };
        this.print = (object) => {
            this.statements.push(`print(${object.__pipcook__identifier})`);
        };
        this.equal = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${object1.__pipcook__identifier} == ${object2.__pipcook__identifier}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        this.notEqual = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${object1.__pipcook__identifier} != ${object2.__pipcook__identifier}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        this.larger = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${object1.__pipcook__identifier} > ${object2.__pipcook__identifier}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        this.smaller = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${object1.__pipcook__identifier} < ${object2.__pipcook__identifier}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        this.largerEqual = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${object1.__pipcook__identifier} >= ${object2.__pipcook__identifier}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        this.smallerEqual = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${object1.__pipcook__identifier} <= ${object2.__pipcook__identifier}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        this.and = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${object1.__pipcook__identifier} and ${object2.__pipcook__identifier}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        this.or = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${object1.__pipcook__identifier} or ${object2.__pipcook__identifier}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        this.not = (object) => {
            const identifier = getId();
            this.statements.push(`${identifier} = not ${object.__pipcook__identifier}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        this.if = (condition, execution) => {
            this.statements.push(`if ${condition.__pipcook__identifier}:`);
            const statementsTemp = this.statements;
            this.statements = [];
            execution();
            const ifStatements = this.statements.map((ele) => '\t' + ele);
            this.statements = [...statementsTemp, ...ifStatements];
        };
        this.while = (condition, execution) => {
            this.statements.push(`while ${condition.__pipcook__identifier}:`);
            const statementsTemp = this.statements;
            this.statements = [];
            execution();
            const ifStatements = this.statements.map((ele) => '\t' + ele);
            this.statements = [...statementsTemp, ...ifStatements];
        };
        this.add = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${object1.__pipcook__identifier} + ${object2.__pipcook__identifier}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        this.minus = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${object1.__pipcook__identifier} - ${object2.__pipcook__identifier}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        this.multiply = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${object1.__pipcook__identifier} * ${object2.__pipcook__identifier}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        this.divide = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${object1.__pipcook__identifier} / ${object2.__pipcook__identifier}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        this.mod = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${object1.__pipcook__identifier} % ${object2.__pipcook__identifier}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        this.pow = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${object1.__pipcook__identifier} ** ${object2.__pipcook__identifier}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
        this.floorDivide = (object1, object2) => {
            const identifier = getId();
            this.statements.push(`${identifier} = ${object1.__pipcook__identifier} // ${object2.__pipcook__identifier}`);
            return python_helper_1.getObject(identifier, this.statements);
        };
    }
    static constructPythonStatements(python) {
        return __awaiter(this, void 0, void 0, function* () {
            const statements = python.statements;
            let codes = '';
            statements.forEach((statement) => {
                codes += statement + '\n';
            });
            yield communication_1.default.execute(python.scope, codes);
            python.statements = [];
        });
    }
    static scope(scope, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            let python = this.pythons.find((e) => e.scope === scope);
            if (!python) {
                python = new Python();
                python.scope = scope;
                this.pythons.push(python);
            }
            yield communication_1.default.openSession(scope);
            callback(python);
            yield this.constructPythonStatements(python);
        });
    }
    fromImport(packageName, importNames) {
        const objects = [];
        importNames.forEach((item) => {
            const identifier = getId();
            this.statements.push(`from ${packageName} import ${item} as ${identifier}`);
            const pythonObject = python_helper_1.getObject(identifier, this.statements);
            objects.push(pythonObject);
        });
        return objects;
    }
}
exports.default = Python;
Python.pythons = [];
Python.exit = (scope) => __awaiter(void 0, void 0, void 0, function* () {
    yield communication_1.default.exit(scope);
});
//# sourceMappingURL=python.js.map